@@ -7,6 +7,7 @@ from .sources import *
 from .namespace import *
 from .when import *
 from .endpoints import *
+from .future import *
 
 
 from .bus import *
@@ -15,6 +16,7 @@ from .pipe import *
 from .monitor import Monitor
 from .page import page
 from .state import *
+from .lambdas import _ as x
 
 try:
     # import panel as pn
@@ -23,4 +25,4 @@ try:
 except ImportError:
     pass
 
-__version__ = '1.0.8'
+__version__ = '1.1.2'
