@@ -10,7 +10,8 @@ from contextlib import closing
 from collections import deque
 import dill
 import json
-
+import threading
+from tornado.ioloop import IOLoop
 
 try:
     import builtins
@@ -29,7 +30,7 @@ __all__ = [
     'dedup', 'uniq', 'to_dataframe', 'pmap', 'pfilter', 'post_to',
     'head', 'read', 'tcp_write', 'write_to_file', 'size', 'ls', 'range',
     'sum', 'split', 'sample', 'extract', 'readlines', 'last',
-    'abs', 'type', 'll',
+    'abs', 'type', 'll', 'pslice',
     'dir', 'help',
     'eval',
     'hash',
@@ -42,9 +43,29 @@ __all__ = [
     'print',
     'range',
     'sum',
+    'get_instances_by_class',
+    'to_json',
+    'cm', 'call_method',
 ]
 
 
+_io_loops = []
+
+
+def get_io_loop(asynchronous=None):
+    if asynchronous:
+        return IOLoop.current()
+
+    if not _io_loops:
+        loop = IOLoop()
+        thread = threading.Thread(target=loop.start)
+        thread.daemon = True
+        thread.start()
+        _io_loops.append(loop)
+
+    return _io_loops[-1]
+
+
 class Pipe:
     """
     Represent a Pipeable Element.
@@ -69,9 +90,17 @@ class Pipe:
         self.func = func
         functools.update_wrapper(self, func)
 
+    def run_async(self, asyncfunc, callback):
+        self.futs = gen.convert_yielded(asyncfunc)
+        self.loop = get_io_loop()
+        self.loop.add_future(self.futs, lambda x: callback(x.result()))
+
     def __ror__(self, other):
         """左边的 |."""
-        return self.func(other)
+        if isinstance(other, gen.Awaitable):
+            self.run_async(other, self.func)
+        else:
+            return self.func(other)
 
     def __rrshift__(self, other):
         """左边的 >>."""
@@ -357,6 +386,13 @@ def split(sep="\n"):
     return _ @ P
 
 
+@Pipe
+def pslice(start, end):
+    def _(iteration):
+        return iteration[start:end]
+    return _ @ P
+
+
 @Pipe
 def attr(name):
     def _(object):
@@ -605,7 +641,7 @@ def size(x):
 
 
 @Pipe
-def post_to(url='http://127.0.0.1:9999', asynchronous=True, headers={}):
+def post_to(url='http://127.0.0.1:7777', asynchronous=True, headers={}):
     """ post a str or bytes or pyobject to url.
 
     str:直接发送
@@ -623,7 +659,7 @@ def post_to(url='http://127.0.0.1:9999', asynchronous=True, headers={}):
     def _encode(body):
         if not isinstance(body, bytes):
             try:
-                body = json.dumps(body)
+                body = json.dumps(body, ensure_ascii=False)
             except TypeError:
                 body = dill.dumps(body)
         return body
@@ -731,15 +767,18 @@ def extract(typ='chinese'):
     import re
 
     url_regex = re.compile(
-        '(?:(?:https?|ftp|file)://|www\.|ftp\.)[-A-Z0-9+&@#/%=~_|$?!:,.]*[A-Z0-9+&@#/%=~_|$]', re.IGNORECASE)
+        r'(?:(?:https?|ftp|file)://|www\.|ftp\.)[-A-Z0-9+&@#/%=~_|$?!:,.]*[A-Z0-9+&@#/%=~_|$]', re.IGNORECASE)
     email_regex = re.compile(
-        '([A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4})', re.IGNORECASE)
+        r'([A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4})', re.IGNORECASE)
 
-    def _(text: str)->list:
+    def _(text: str) -> list:
         if typ == 'chinese':
             return re.findall(r"[\u4e00-\u9fa5]+", text)
         elif typ == 'numbers' or typ == 'number':
-            return re.findall("[+-]?\d+\.*\d+", text) >> pmap(lambda x: float(x) if '.' in x else int(x)) >> ls
+            return re.findall(
+                r"[+-]?\d+\.*\d*", text
+            ) >> pmap(lambda x:
+                      float(x) if '.' in x else int(x)) >> ls
         elif typ == 'table':
             import pandas as pd
             return pd.read_html(text)
@@ -754,6 +793,14 @@ def extract(typ='chinese'):
     return _ @ P
 
 
+@P
+def get_instances_by_class(cls):
+    import gc
+    for obj in gc.get_objects():
+        if isinstance(obj, cls):
+            yield obj
+
+
 # %%转换内置函数为pipe
 for i in builtins.__dict__.copy():
     if callable(builtins.__dict__.get(i)):
@@ -791,6 +838,35 @@ to_str = P(str)
 # 这种情况会导致isinstanced等非直接调用方法失败
 
 
+@P
+def to_json(r):
+    if hasattr(r, 'json'):
+        return r.json()
+    elif hasattr(r, 'to_json'):
+        return r.to_json()
+    else:
+        return json.loads(r)
+
+
+@Pipe
+def cm(mkey):
+    "call method by a method key"
+
+    def _cm(obj):
+        for method in dir(obj):
+            if callable(getattr(obj, method)) and mkey in method:
+                return getattr(obj, method)()
+
+    if isinstance(mkey, str):
+        return _cm @ P
+    else:
+        obj, mkey = mkey, 'json'
+        return _cm(obj)
+
+
+call_method = cm
+
+
 if __name__ == "__main__":
     import doctest
     doctest.testfile('README.md')
