@@ -1,7 +1,7 @@
 import subprocess
 import json
 from tornado.web import RequestHandler, Application
-from tornado.httpserver import HTTPServer
+# from tornado.httpserver import HTTPServer
 from tornado import gen
 from tornado.tcpserver import TCPServer
 from tornado.tcpclient import TCPClient
@@ -13,6 +13,7 @@ import tornado.ioloop
 
 from .topic import RedisStream
 from .core import Stream
+from .namespace import NB
 
 import logging
 
@@ -32,19 +33,6 @@ def PeriodicCallback(callback, callback_time, asynchronous=False, **kwargs):
     return source
 
 
-def sink_to_file(filename, upstream, mode='w',
-                 prefix='', suffix='\n', flush=False):
-    file = open(filename, mode=mode)
-
-    def write(text):
-        file.write(prefix + text + suffix)
-        if flush:
-            file.flush()
-
-    upstream.sink(write)
-    return file
-
-
 class Source(Stream):
     _graphviz_shape = 'doubleoctagon'
 
@@ -288,6 +276,12 @@ class from_http_server(Source):
                 yield self.source._emit(self.request.body)
                 self.write('OK')
 
+            @gen.coroutine
+            def get(self):
+                response = self.get_arguments("question")
+                yield self.source._emit(response)
+                self.write('OK')
+
         application = Application([
             (self.path, Handler),
         ])
@@ -526,7 +520,7 @@ class from_redis(Stream):
 class from_http_request(Stream):
     """Receive data from http request,emit httprequest data to stream."""
 
-    def __init__(self, port, path='/.*', start=False, server_kwargs=None):
+    def __init__(self, port=7777, path='/.*', start=False, server_kwargs=None):
         self.port = port
         self.path = path
         self.server_kwargs = server_kwargs or {}
@@ -538,6 +532,8 @@ class from_http_request(Stream):
 
     def _start_server(self):
         class Handler(RequestHandler):
+            source = self
+
             def _loads(self, body):
                 """解析从web端口提交过来的数据.
 
@@ -554,24 +550,23 @@ class from_http_request(Stream):
                 finally:
                     return body
 
-            source = self
-
             @gen.coroutine
             def post(self):
                 self.request.body = self._loads(self.request.body)
                 yield self.source._emit(self.request.body)
                 self.write('OK')
 
-        application = Application([
+        self.application = Application([
             (self.path, Handler),
         ])
-        self.server = HTTPServer(application, **self.server_kwargs)
-        self.server.listen(self.port)
+        # self.server = HTTPServer(application, **self.server_kwargs)
+        self.server = self.application.listen(self.port)
 
     def start(self):
         if self.stopped:
             self.stopped = False
-            self.loop.add_callback(self._start_server)
+            self._start_server()
+            # self.loop.add_callback(self._start_server)#这个会导致在端口占用情况下不报错
 
     def stop(self):
         """Shutdown HTTP server."""
@@ -675,6 +670,58 @@ class StreamTCPClient():
             self.stopped = True
 
 
+@Stream.register_api(staticmethod)
+class from_mail(Source):
+    def __init__(self, host=None,
+                 username=None,
+                 password=None,
+                 ssl=True,
+                 ssl_context=None,
+                 starttls=False,
+                 interval=60*15, start=False, **kwargs):
+        from imbox import Imbox
+        if not host:
+            try:
+                hostname = NB('mail')['hostname']
+                username = NB('mail')['username']
+                password = NB('mail')['password']
+            except:
+                raise('no host username password ')
+        self.imbox = Imbox(hostname,
+                           username=username,
+                           password=password,
+                           ssl=True,
+                           ssl_context=None,
+                           starttls=False)
+
+        self.interval = interval
+        super(from_mail, self).__init__(ensure_io_loop=True, **kwargs)
+        self.stopped = True
+        if start:
+            self.start()
+
+    @gen.coroutine
+    def poll_mail(self):
+        while True:
+            for uid, msg in self.imbox.messages(unread=True):
+                self._emit(msg)
+                self.imbox.mark_seen(uid)
+            yield gen.sleep(self.interval)
+            if self.stopped:
+                break
+
+    def start(self):
+        if self.stopped:
+            self.stopped = False
+            self.loop.add_callback(self.poll_mail)
+
+    def stop(self):
+        self.stopped = True
+
+    def logout(self):
+        self.imbox.logout()
+
+
 def gen_block_test() -> int:
     import time
     import datetime
